# -*- coding: utf-8 -*-
"""
Created on Mon May 30 18:36:21 2016

@author: daniel weiß
"""

# Tested with Python 3.5.
# This programm generates Figure 2 of the preprint
# "Metastable energy strata in numerical discretizations
# of weakly nonlinear wave equations"
# written by Ludwig Gauckler and Daniel Weiss. 

import numpy as np
import matplotlib.pyplot as plt

# parameter of wave equation
rho = 0.5

# quadratic nonlinearity of wave equation
def g(u,N):
    u_shift = np.fft.ifftshift(u)
    u_fft = np.fft.fft(u_shift)
    v_shift = np.fft.ifft(u_fft*u_fft)
    return np.fft.fftshift(v_shift)


# trigonometric integrator
def trigo(tau, t0, tend, pn, qn, Omega, N):
    # number of steps, 
    steps = round((tend - t0)/tau)
    steps1 = round(1200/tau)
    steps2 = round(1400/tau)
    # in python: sinc(x)=sin(x*pi)/(x*pi)
    sinOt, cosOt = np.sin(Omega*tau), np.cos(Omega*tau)
    sincOt = np.sinc(Omega*tau/np.pi)
    td2 = tau/2
    t2d2sincOt = tau*td2*sincOt
    td2cosOt = td2*cosOt
    sinOtdOmega = sinOt/Omega
    OmegasinOt = -Omega*sinOt

    gn = g(qn, N)

    # initializing solution-array
    Q = np.zeros((steps1+steps2+2, N), dtype=complex)
    P = np.zeros((steps1+steps2+2, N), dtype=complex)
    Q[0, :] = qn
    P[0, :] = pn

    ksol = 1
    for k in range(1, steps+1):
        qn1 = cosOt*qn + sinOtdOmega*pn + t2d2sincOt*gn
        gn1 = g(qn1, N)
        pn = OmegasinOt*qn + cosOt*pn + td2cosOt*gn + td2*gn1
        gn = gn1
        qn = qn1
        if k <= steps1 or k >= steps-steps2:
            Q[ksol, :] = qn
            P[ksol, :] = pn
            ksol += 1

    return P, Q


# plot of solution
def plotsol(t0, tend, P, Q, M):
    for l in range(M, M+9):
        energy = 1.0/2*np.abs(Q[:, l]*Omega[l])**2 + 1.0/2*np.abs(P[:, l])**2
        plt.semilogy(np.linspace(t0+tend-1200, tend, len(energy)), energy)
    plt.tick_params(labelsize=20)
    plt.ylim(1e-25, 1e-1)
    plt.yticks([1e-3, 1e-8, 1e-13, 1e-18, 1e-23])

        

# intervall of integration
t0,  tend = 0,  1.0e6+200

# discretization parameter of spectral methods
M = 2**5
N = 2*M

# frequencies
Omega = np.sqrt(np.arange(-M, M)**2+rho)

# initial values
epsilon = 1e-3
p0 = np.zeros(N, dtype=complex)
q0 = np.zeros(N, dtype=complex)
# M is the index of the zero-frequency component
p0[M-1] = np.sqrt(epsilon) * np.exp(2j)
p0[M+1] = p0[M-1].conjugate()
q0[M-1] = np.sqrt(epsilon)/Omega[M-1] * np.exp(3j)
q0[M+1] = q0[M-1].conjugate()


# step-size of trigonometric integrator of Figure 2
tau = 0.45
steps1 = round(1200/tau)
steps2 = round(1400/tau)
# numerical integration
P, Q = trigo(tau, t0, tend, p0, q0, Omega, N)
# plot energies
plt.figure("Figure 2 (left)")
plotsol(t0, 1200, P[:steps1, :], Q[:steps1, :], M)
plt.xticks([0, 1000])
plt.figure("Figure 2 (right)")
plotsol(-200, tend, P[-steps2:, :], Q[-steps2:, :], M)
plt.xticks([1e6-1000, 1e6]) 

plt.show()
