# -*- coding: utf-8 -*-
"""
Created on Mon May 30 18:36:21 2016

@author: daniel weiß
"""
# Tested with Python 3.5.
# This programm generates Figure 3 of the preprint
# "Metastable energy strata in numerical discretizations
# of weakly nonlinear wave equations"
# written by Ludwig Gauckler and Daniel Weiss. 

import numpy as np
import matplotlib.pyplot as plt

# parameter of wave equation
rho = 0.5

# quadratic nonlinearity of wave equation
def g(u, N):
    u_shift = np.fft.ifftshift(u)
    u_fft = np.fft.fft(u_shift)
    v_shift = np.fft.ifft(u_fft*u_fft)
    return np.fft.fftshift(v_shift)


# trigonometric integrator
def trigo(tau, t0, tend, pn, qn, Omega, N):
    # number of steps
    steps = int(round((tend - t0)/tau))
    # in python: sinc(x)=sin(x*pi)/(x*pi)
    sinOt, cosOt = np.sin(Omega*tau), np.cos(Omega*tau)
    sincOt = np.sinc(Omega*tau/np.pi)
    td2 = tau/2
    t2d2sincOt = tau*td2*sincOt
    td2cosOt = td2*cosOt
    sinOtdOmega = sinOt/Omega
    OmegasinOt = -Omega*sinOt

    gn = g(qn, N)

    # initializing solution-array
    Q = np.zeros((steps+1, N), dtype=complex)
    P = np.zeros((steps+1, N), dtype=complex)
    Q[0, :] = qn
    P[0, :] = pn

    for k in range(1, steps+1):
        qn1 = cosOt*qn + sinOtdOmega*pn + t2d2sincOt*gn
        gn1 = g(qn1, N)
        pn = OmegasinOt*qn + cosOt*pn + td2cosOt*gn + td2*gn1
        gn = gn1
        qn = qn1
        Q[k, :] = qn
        P[k, :] = pn

    return P, Q


# plot of solution
def plotsol(tau, t0, tend, P, Q, M, N):
    steps = int(round((tend - t0)/tau))
    maxenergy = np.zeros(np.floor(steps/100))
    for l in range(M, N):
        energy = 1.0/2*np.abs(Q[:, l]*Omega[l])**2 + 1.0/2*np.abs(P[:, l])**2
        for k in range(0,len(maxenergy)):
            maxenergy[k] = max(energy[k*100:(k+1)*100-1])
        plt.semilogy(np.linspace(t0, tend, len(maxenergy)), maxenergy)
    plt.tick_params(labelsize=20)
    plt.ylim(1e-25, 1e-1)
    plt.xlim(t0,tend)
    plt.yticks([1e-3, 1e-8, 1e-13, 1e-18, 1e-23])
    plt.xticks([0, 10000])



# intervall of integration
t0,  tend = 0,  1.3e4

# discretization parameter of spectral methods
M = 2**5
N = 2*M

# frequencies
Omega = np.sqrt(np.arange(-M, M)**2+rho)

# initial values
epsilon = 1e-3
p0 = np.zeros(N, dtype=complex)
q0 = np.zeros(N, dtype=complex)
# M is the index of the zero-frequency component
p0[M-1] = np.sqrt(epsilon) * np.exp(2j)
p0[M+1] = p0[M-1].conjugate()
q0[M-1] = np.sqrt(epsilon)/Omega[M-1] * np.exp(3j)
q0[M+1] = q0[M-1].conjugate()


# step-size of trigonometric integrator of Figure 3 (left)
tau = 2*np.pi/(Omega[M+1]+Omega[M+6]+Omega[M+7])
# numerical integration
P, Q = trigo(tau, t0, tend, p0, q0, Omega, N)
# plot energies
plt.figure("Figure 3 (left)")
plotsol(tau, t0, tend, P, Q, M, N)


# step-size of trigonometric integrator of Figure 3 (right)
tau = 2*np.pi/(-Omega[M+1]+Omega[M+6]+Omega[M+7])
# numerical integration
P, Q = trigo(tau, t0, tend, p0, q0, Omega, N)
# plot energies
plt.figure("Figure 3 (right)")
plotsol(tau, t0, tend, P, Q, M, N)

plt.show()
